public class Galaxy {

/*
**********************************************************************
Class Variables
**********************************************************************
*/

   double galmass;
   double ahalo;
   double vhalo;
   double rthalo;
   Triple galpos = new Triple();
   Triple galvel = new Triple();
   Triple galacc = new Triple();

/*
**********************************************************************
Constructor(s)
**********************************************************************
*/

   Galaxy(double massin, double ahaloin, double vhaloin, double rthaloin) {

      galmass=massin;
      ahalo=ahaloin;
      vhalo=vhaloin;
      rthalo=rthaloin;
      galpos.x=0.0;
      galpos.y=0.0;
      galpos.z=0.0;
      galvel.x=0.0;
      galvel.y=0.0;
      galvel.z=0.0;
   }

   Galaxy() {

   }

/*
**********************************************************************
setPosvel
**********************************************************************
*/
      
   public void setPosvel(Triple pos, Triple vel) {

      galpos = pos;
      galvel = vel;

   }

/*
**********************************************************************
scaleMass  
**********************************************************************
*/

   public void scaleMass(double massFact) {

      double a2,a1,a0,q,r,s1,s2;

      galmass = 4.8*massFact;
      vhalo = 1.0*Math.pow(massFact,0.25);
      ahalo = 0.1*Math.pow(massFact,0.5);


      a2 = -galmass/vhalo/vhalo;
      a1 = -2.*ahalo*galmass/vhalo/vhalo;
      a0 = -galmass*ahalo*ahalo/vhalo/vhalo;
      q = a1/3.0 - a2*a2/9.0;
      r = (a1*a2-3.*a0)/6.0 - a2*a2*a2/27.0;

      s1 = Math.pow((r + Math.sqrt(q*q*q+r*r)),0.333);
      s2 = Math.pow((r - Math.sqrt(q*q*q+r*r)),0.333);

      rthalo=(s1+s2)-a2/3;

   }

/*
**********************************************************************
MoveGalaxy
**********************************************************************
*/

   public void MoveGalaxy(double dtime) {

      Triple newpos = new Triple();
      Triple newvel = new Triple();

      newpos.x = galpos.x + galvel.x * dtime + 0.5f * galacc.x * dtime*dtime;
      newpos.y = galpos.y + galvel.y * dtime + 0.5f * galacc.y * dtime*dtime;
      newpos.z = galpos.z + galvel.z * dtime + 0.5f * galacc.z * dtime*dtime;

      newvel.x = galvel.x + galacc.x * dtime;
      newvel.y = galvel.y + galacc.y * dtime;
      newvel.z = galvel.z + galacc.z * dtime;

      galpos = newpos;
      galvel = newvel;

   }

/*
**********************************************************************
Acceleration
**********************************************************************
*/

   public Triple Acceleration(Triple posin) {

      final double G=1.0;
      double dx,dy,dz;
      double r,AccMag;
      Triple calcacc=new Triple();

      dx=posin.x-galpos.x;
      dy=posin.y-galpos.y;
      dz=posin.z-galpos.z;
      r=Math.sqrt(dx*dx+dy*dy+dz*dz);
      AccMag=-G*InteriorMass(r)/(r*r);
      
      calcacc.x=dx*AccMag/r;
      calcacc.y=dy*AccMag/r;
      calcacc.z=dz*AccMag/r;

      return calcacc;

   }

/*
**********************************************************************
Potential   
**********************************************************************
*/

   public double Potential(Triple posin) {

      final double G=1.0;
      double dx,dy,dz;
      double r,pot;

     dx=posin.x-galpos.x;
     dy=posin.y-galpos.y;
     dz=posin.z-galpos.z;
      r=Math.sqrt(dx*dx+dy*dy+dz*dz);
      pot=G*InteriorMass(r)/r;

      return pot;

   }

/*
**********************************************************************
InteriorMass
**********************************************************************
*/
   public double InteriorMass(double r) {

      double intmass;

      if (r < rthalo)
        intmass=vhalo*vhalo*r*r*r/((ahalo+r)*(ahalo+r));
      else
        intmass=galmass;

      return intmass;

   }

/*
**********************************************************************
Density
**********************************************************************
*/
   public double Density(double r) {

      double density, rinner, router;
      double minner, mouter, dm, vol;

// a stupid way
      rinner=r*0.99;
      router=r*1.01;
      minner=InteriorMass(r*0.99);
      mouter=InteriorMass(r*1.01);
      dm=(mouter-minner);
      vol=(4./3.)*Math.PI*(Math.pow(router,3)-Math.pow(rinner,3));
      density=dm/vol;

      return density;

   }

/*
**********************************************************************
DynFric -- sort of mimics a Chandrasekhar-type expression
**********************************************************************
*/
   public Triple DynFric(double pmass, Triple ppos, Triple pvel) {

      final double G=1.0;
      double r,v,galrho,fricmag;
      double f=0.5;
      double lnGamma=3.0;
      Triple friction = new Triple();
      Triple dr = new Triple();
      Triple dv = new Triple();

      dv=Triple.SubTriple(pvel,galvel);
      v=Triple.TripleMag(dv);
      dr=Triple.SubTriple(ppos,galpos);
      r=Triple.TripleMag(dr);
	 
      galrho=Density(r);
      fricmag=4.0*Math.PI*G*lnGamma*pmass*galrho*v/Math.pow((1+v),3);

      friction.x = -(dv.x/v)*fricmag;
      friction.y = -(dv.y/v)*fricmag;
      friction.z = -(dv.z/v)*fricmag;

      return friction;

   }

/*
**********************************************************************
PrintGalaxy 
**********************************************************************
*/

   public void PrintGalaxy() {

      System.out.println("Mass: " + galmass + "  rthalo: " + rthalo);
      System.out.println("Pos : " + galpos.x + "," + galpos.y + "," + galpos.z); 
      System.out.println("Vel : " + galvel.x + "," + galvel.y + "," + galvel.z); 
      System.out.println("Acc : " + galacc.x + "," + galacc.y + "," + galacc.z); 

   }

}
